/// <reference types="@girs/cairo-1.0/ambient"/>
import GLib from 'gi://GLib';
import Meta from 'gi://Meta';
import ClutterImpl from 'gi://Clutter';
import CairoImpl from 'gi://cairo';
import * as Main from "resource:///org/gnome/shell/ui/main.js";
import { Rect } from "./rect.js";
import { Point } from './point.js';
export var MaximizeFlags;
(function (MaximizeFlags) {
    MaximizeFlags[MaximizeFlags["Neither"] = 0] = "Neither";
})(MaximizeFlags || (MaximizeFlags = {}));
;
export var GnomeSystem;
(function (GnomeSystem) {
    GnomeSystem.Clutter = ClutterImpl;
    GnomeSystem.Cairo = CairoImpl;
    // TODO can we do less type casting?
    function newClutterActor() {
        return new ClutterImpl.Actor();
    }
    GnomeSystem.newClutterActor = newClutterActor;
    function newClutterCanvas() {
        return new ClutterImpl.Canvas();
    }
    GnomeSystem.newClutterCanvas = newClutterCanvas;
    function newClutterColor(components) {
        // generated types are wrong, it takes obj with { red, gren, blue} props not separate args
        const Color = ClutterImpl.Color;
        return new Color(components);
    }
    GnomeSystem.newClutterColor = newClutterColor;
    function numWorkspaces() {
        return global.workspace_manager.get_n_workspaces();
    }
    GnomeSystem.numWorkspaces = numWorkspaces;
    function workspaceIndex() {
        return global.workspace_manager.get_active_workspace_index();
    }
    GnomeSystem.workspaceIndex = workspaceIndex;
    function activateWorkspace(idx) {
        let ws = global.workspace_manager.get_workspace_by_index(idx);
        ws.activate(global.get_current_time());
    }
    GnomeSystem.activateWorkspace = activateWorkspace;
    function moveWindowToWorkspace(win, idx) {
        let ws = global.workspace_manager.get_workspace_by_index(idx);
        win.change_workspace_by_index(idx, false);
        ws.activate_with_focus(win, global.get_current_time());
    }
    GnomeSystem.moveWindowToWorkspace = moveWindowToWorkspace;
    function stableSequence(win) {
        return win.get_stable_sequence();
    }
    GnomeSystem.stableSequence = stableSequence;
    function windowTitle(win) {
        return win.get_title();
    }
    GnomeSystem.windowTitle = windowTitle;
    function rect(metaRect) {
        return {
            pos: { x: metaRect.x, y: metaRect.y },
            size: { x: metaRect.width, y: metaRect.height },
        };
    }
    function size(metaRect) {
        return { x: metaRect.width, y: metaRect.height };
    }
    function windowRect(win) {
        let globalRect = rect(win.get_frame_rect());
        return Rect.move(globalRect, Point.scaleConstant(-1, workspaceOffset(win)));
    }
    GnomeSystem.windowRect = windowRect;
    function minimize(win) {
        win.minimize();
    }
    GnomeSystem.minimize = minimize;
    function unminimize(win) {
        win.unminimize();
    }
    GnomeSystem.unminimize = unminimize;
    function moveResize(win, workspaceRect) {
        if (win.get_maximized() !== MaximizeFlags.Neither) {
            unmaximize(win);
        }
        let rect = Rect.move(workspaceRect, workspaceOffset(win));
        win.move_resize_frame(true, rect.pos.x, rect.pos.y, rect.size.x, rect.size.y);
    }
    GnomeSystem.moveResize = moveResize;
    function currentWindow() {
        return global.display['focus-window'];
    }
    GnomeSystem.currentWindow = currentWindow;
    function workspaceOffset(win) {
        return rect(win.get_work_area_current_monitor()).pos;
    }
    GnomeSystem.workspaceOffset = workspaceOffset;
    function translateEventCoordinates(point, win) {
        return Point.subtract(point, workspaceOffset(win));
    }
    GnomeSystem.translateEventCoordinates = translateEventCoordinates;
    function workspaceArea(win) {
        return size(win.get_work_area_current_monitor());
    }
    GnomeSystem.workspaceArea = workspaceArea;
    function unmaximize(win) {
        win.unmaximize(Meta.MaximizeFlags.VERTICAL | Meta.MaximizeFlags.HORIZONTAL);
    }
    GnomeSystem.unmaximize = unmaximize;
    function maximize(win) {
        win.maximize(Meta.MaximizeFlags.VERTICAL | Meta.MaximizeFlags.HORIZONTAL);
    }
    GnomeSystem.maximize = maximize;
    function getMaximized(win) {
        return win.get_maximized() !== MaximizeFlags.Neither;
    }
    GnomeSystem.getMaximized = getMaximized;
    const VISIBLE_WINDOW_TYPES = [
        Meta.WindowType.NORMAL,
        Meta.WindowType.DIALOG,
        Meta.WindowType.UTILITY,
    ];
    function listWindows(win) {
        const display = (win == null) ? global.display : win.get_display();
        const screenNo = (win == null) ? display.get_current_monitor() : win.get_monitor();
        let windows = display.get_workspace_manager().get_active_workspace().list_windows();
        return windows.filter(function (w) {
            return (w.get_monitor() == screenNo
                && VISIBLE_WINDOW_TYPES.indexOf(w.get_window_type()) !== -1);
        });
    }
    function visibleWindows() {
        const current = currentWindow();
        let windows = listWindows(current).filter(function (w) {
            return !w.minimized;
        });
        return [current, windows];
    }
    GnomeSystem.visibleWindows = visibleWindows;
    function minimizedWindows() {
        return listWindows().filter(function (w) {
            return w.minimized;
        });
    }
    GnomeSystem.minimizedWindows = minimizedWindows;
    function activate(win) {
        Main.activateWindow(win, global.get_current_time());
    }
    GnomeSystem.activate = activate;
    function pushModal(actor) {
        return Main.pushModal(actor);
    }
    GnomeSystem.pushModal = pushModal;
    function popModal(grab) {
        Main.popModal(grab);
    }
    GnomeSystem.popModal = popModal;
    function activateLater(win) {
        GLib.timeout_add(GLib.PRIORITY_DEFAULT, 10, function () {
            Main.activateWindow(win, global.get_current_time());
            return false;
        });
    }
    GnomeSystem.activateLater = activateLater;
    function setWindowHidden(win, hidden) {
        let actor = win.get_compositor_private();
        if (hidden) {
            actor.hide();
        }
        else {
            actor.show();
        }
    }
    GnomeSystem.setWindowHidden = setWindowHidden;
})(GnomeSystem || (GnomeSystem = {}));
